@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,18 +15,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_IRC_SERVER_H
-#define WEECHAT_IRC_SERVER_H 1
+#ifndef WEECHAT_PLUGIN_IRC_SERVER_H
+#define WEECHAT_PLUGIN_IRC_SERVER_H
 
+#include <time.h>
 #include <sys/time.h>
 #include <regex.h>
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
-#endif /* HAVE_GNUTLS */
 
 #ifndef NI_MAXHOST
 #define NI_MAXHOST 256
@@ -41,6 +40,13 @@ enum t_irc_server_sasl_fail
     IRC_SERVER_NUM_SASL_FAIL,
 };
 
+enum t_irc_server_charset_message
+{
+    IRC_SERVER_CHARSET_MESSAGE_MESSAGE = 0,
+    IRC_SERVER_CHARSET_MESSAGE_CHANNEL,
+    IRC_SERVER_CHARSET_MESSAGE_TEXT,
+};
+
 enum t_irc_server_option
 {
     IRC_SERVER_OPTION_ADDRESSES = 0, /* server addresses (IP/name with port) */
@@ -48,6 +54,7 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_IPV6,          /* use IPv6 protocol                    */
     IRC_SERVER_OPTION_SSL,           /* SSL protocol                         */
     IRC_SERVER_OPTION_SSL_CERT,      /* client ssl certificate file          */
+    IRC_SERVER_OPTION_SSL_PASSWORD,   /* client ssl certificate key password */
     IRC_SERVER_OPTION_SSL_PRIORITIES, /* gnutls priorities                   */
     IRC_SERVER_OPTION_SSL_DHKEY_SIZE, /* Diffie Hellman key size             */
     IRC_SERVER_OPTION_SSL_FINGERPRINT, /* SHA1 fingerprint of certificate    */
@@ -57,16 +64,18 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_SASL_MECHANISM,/* mechanism for SASL authentication    */
     IRC_SERVER_OPTION_SASL_USERNAME, /* username for SASL authentication     */
     IRC_SERVER_OPTION_SASL_PASSWORD, /* password for SASL authentication     */
-    IRC_SERVER_OPTION_SASL_KEY,      /* key for ECDSA-NIST256P-CHALLENCE     */
+    IRC_SERVER_OPTION_SASL_KEY,      /* key for ECDSA-NIST256P-CHALLENGE     */
     IRC_SERVER_OPTION_SASL_TIMEOUT,  /* timeout for SASL authentication      */
     IRC_SERVER_OPTION_SASL_FAIL,     /* action on SASL fail                  */
     IRC_SERVER_OPTION_AUTOCONNECT,   /* autoconnect to server at startup     */
     IRC_SERVER_OPTION_AUTORECONNECT, /* autoreconnect when disconnected      */
     IRC_SERVER_OPTION_AUTORECONNECT_DELAY, /* delay before trying again reco */
     IRC_SERVER_OPTION_NICKS,         /* nicknames (comma separated list)     */
+    IRC_SERVER_OPTION_NICKS_ALTERNATE, /* use alternate nicknames            */
     IRC_SERVER_OPTION_USERNAME,      /* user name                            */
     IRC_SERVER_OPTION_REALNAME,      /* real name                            */
     IRC_SERVER_OPTION_LOCAL_HOSTNAME,/* custom local hostname                */
+    IRC_SERVER_OPTION_USERMODE,      /* usermode to set once connected       */
     IRC_SERVER_OPTION_COMMAND,       /* command to run once connected        */
     IRC_SERVER_OPTION_COMMAND_DELAY, /* delay after execution of command     */
     IRC_SERVER_OPTION_AUTOJOIN,      /* channels to automatically join       */
@@ -77,10 +86,13 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_ANTI_FLOOD_PRIO_LOW,  /* anti-flood (low priority)     */
     IRC_SERVER_OPTION_AWAY_CHECK,           /* delay between away checks     */
     IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS, /* max nicks for away check      */
-    IRC_SERVER_OPTION_DEFAULT_MSG_KICK,     /* default kick message          */
-    IRC_SERVER_OPTION_DEFAULT_MSG_PART,     /* default part message          */
-    IRC_SERVER_OPTION_DEFAULT_MSG_QUIT,     /* default quit message          */
+    IRC_SERVER_OPTION_MSG_KICK,             /* default kick message          */
+    IRC_SERVER_OPTION_MSG_PART,             /* default part message          */
+    IRC_SERVER_OPTION_MSG_QUIT,             /* default quit message          */
     IRC_SERVER_OPTION_NOTIFY,               /* notify list                   */
+    IRC_SERVER_OPTION_SPLIT_MSG_MAX_LENGTH, /* max length of messages        */
+    IRC_SERVER_OPTION_CHARSET_MESSAGE,      /* what to decode/encode in msg  */
+    IRC_SERVER_OPTION_DEFAULT_CHANTYPES,    /* chantypes if not received     */
     /* number of server options */
     IRC_SERVER_NUM_OPTIONS,
 };
@@ -114,9 +126,12 @@ enum t_irc_server_option
 #define IRC_SERVER_NUM_OUTQUEUES_PRIO 2
 
 /* flags for irc_server_sendf() */
-#define IRC_SERVER_SEND_OUTQ_PRIO_HIGH   1
-#define IRC_SERVER_SEND_OUTQ_PRIO_LOW    2
-#define IRC_SERVER_SEND_RETURN_HASHTABLE 4
+#define IRC_SERVER_SEND_OUTQ_PRIO_HIGH   (1 << 0)
+#define IRC_SERVER_SEND_OUTQ_PRIO_LOW    (1 << 1)
+#define IRC_SERVER_SEND_RETURN_HASHTABLE (1 << 2)
+
+/* version strings */
+#define IRC_SERVER_VERSION_CAP "302"
 
 /* casemapping (string comparisons for nicks/channels) */
 enum t_irc_server_casemapping
@@ -128,6 +143,15 @@ enum t_irc_server_casemapping
     IRC_SERVER_NUM_CASEMAPPING,
 };
 
+/* utf8mapping (string comparisons for nicks/channels) */
+enum t_irc_server_utf8mapping
+{
+    IRC_SERVER_UTF8MAPPING_NONE = 0,
+    IRC_SERVER_UTF8MAPPING_RFC8265,
+    /* number of utf8mapping */
+    IRC_SERVER_NUM_UTF8MAPPING,
+};
+
 /* output queue of messages to server (for sending slowly to server) */
 
 struct t_irc_outqueue
@@ -150,8 +174,10 @@ struct t_irc_server
 
     /* internal vars */
     int temp_server;                /* temporary server (not saved)          */
+    int fake_server;                /* fake server (no I/O, for tests)       */
     int reloading_from_config;      /* 1 if reloading from config file       */
     int reloaded_from_config;       /* 1 if reloaded from config file        */
+    char *addresses_eval;           /* evaluation of addresses               */
     int addresses_count;            /* number of addresses                   */
     char **addresses_array;         /* addresses (after split)               */
     int *ports_array;               /* ports for addresses                   */
@@ -167,14 +193,18 @@ struct t_irc_server
     struct t_hook *hook_fd;         /* hook for server socket                */
     struct t_hook *hook_timer_connection; /* timer for connection            */
     struct t_hook *hook_timer_sasl; /* timer for SASL authentication         */
+    char *sasl_scram_client_first;  /* first message sent for SASL SCRAM     */
+    char *sasl_scram_salted_pwd;    /* salted password for SASL SCRAM        */
+    int sasl_scram_salted_pwd_size; /* size of salted password for SASL SCRAM*/
+    char *sasl_scram_auth_message;  /* auth message for SASL SCRAM           */
+    char *sasl_temp_username;       /* temp SASL username (set by /auth cmd) */
+    char *sasl_temp_password;       /* temp SASL password (set by /auth cmd) */
     int is_connected;               /* 1 if WeeChat is connected to server   */
     int ssl_connected;              /* = 1 if connected with SSL             */
     int disconnected;               /* 1 if server has been disconnected     */
-#ifdef HAVE_GNUTLS
     gnutls_session_t gnutls_sess;   /* gnutls session (only if SSL is used)  */
     gnutls_x509_crt_t tls_cert;     /* certificate used if ssl_cert is set   */
     gnutls_x509_privkey_t tls_cert_key; /* key used if ssl_cert is set       */
-#endif /* HAVE_GNUTLS */
     char *unterminated_message;     /* beginning of a message in input buf   */
     int nicks_count;                /* number of nicknames                   */
     char **nicks_array;             /* nicknames (after split)               */
@@ -184,18 +214,28 @@ struct t_irc_server
                                     /* (nick____1, nick____2, ...)           */
     char *nick;                     /* current nickname                      */
     char *nick_modes;               /* nick modes                            */
-    int cap_away_notify;            /* 1 if capability away-notify is enabled*/
-    int cap_account_notify;         /* 1 if CAP account-notify is enabled    */
+    char *host;                     /* self host (set if known)              */
+    int checking_cap_ls;            /* 1 if checking supported capabilities  */
+    struct t_hashtable *cap_ls;     /* list of supported capabilities        */
+    int checking_cap_list;          /* 1 if checking enabled capabilities    */
+    struct t_hashtable *cap_list;   /* list of enabled capabilities          */
     char *isupport;                 /* copy of message 005 (ISUPPORT)        */
     char *prefix_modes;             /* prefix modes from msg 005 (eg "ohv")  */
     char *prefix_chars;             /* prefix chars from msg 005 (eg "@%+")  */
     int nick_max_length;            /* max length of nick (from msg 005)     */
+    int user_max_length;            /* max length of user (from msg 005)     */
+    int host_max_length;            /* max length of host (from msg 005)     */
     int casemapping;                /* casemapping from msg 005              */
+    int utf8mapping;                /* utf8mapping from msg 005              */
     char *chantypes;                /* chantypes from msg 005 (eg "&#")      */
     char *chanmodes;                /* chanmodes from msg 005                */
                                     /* (eg "beI,k,l,imnpstaqr")              */
     int monitor;                    /* monitor limit from msg 005 (eg 100)   */
     time_t monitor_time;            /* time for monitoring nicks (on connect)*/
+    char *clienttagdeny;            /* list of blocked client-only tags      */
+    int clienttagdeny_count;        /* number of masks in clienttagdeny      */
+    char **clienttagdeny_array;     /* masks expanded from clienttagdeny     */
+    int typing_allowed;             /* typing not excluded by clienttagdeny? */
     int reconnect_delay;            /* current reconnect delay (growing)     */
     time_t reconnect_start;         /* this time + delay = reconnect time    */
     time_t command_time;            /* this time + command_delay = time to   */
@@ -244,7 +284,6 @@ struct t_irc_message
 
 /* digest algorithms for fingerprint */
 
-#ifdef HAVE_GNUTLS
 enum t_irc_fingerprint_digest_algo
 {
     IRC_FINGERPRINT_ALGO_SHA1 = 0,
@@ -253,13 +292,13 @@ enum t_irc_fingerprint_digest_algo
     /* number of digest algorithms */
     IRC_FINGERPRINT_NUM_ALGOS,
 };
-#endif /* HAVE_GNUTLS */
 
+extern char *irc_server_prefix_modes_default;
+extern char *irc_server_prefix_chars_default;
+extern char *irc_server_chanmodes_default;
 extern struct t_irc_server *irc_servers;
-#ifdef HAVE_GNUTLS
 extern const int gnutls_cert_type_prio[];
 extern const int gnutls_prot_prio[];
-#endif /* HAVE_GNUTLS */
 extern struct t_irc_message *irc_recv_msgq, *irc_msgq_last_msg;
 extern char *irc_server_sasl_fail_string[];
 extern char *irc_server_options[][2];
@@ -269,22 +308,38 @@ extern struct t_irc_server *irc_server_search (const char *server_name);
 extern struct t_irc_server *irc_server_casesearch (const char *server_name);
 extern int irc_server_search_option (const char *option_name);
 extern int irc_server_search_casemapping (const char *casemapping);
+extern int irc_server_search_utf8mapping (const char *utf8mapping);
 extern int irc_server_strcasecmp (struct t_irc_server *server,
                                   const char *string1, const char *string2);
 extern int irc_server_strncasecmp (struct t_irc_server *server,
                                    const char *string1, const char *string2,
                                    int max);
+extern char *irc_server_eval_expression (struct t_irc_server *server,
+                                         const char *string);
+extern void irc_server_sasl_get_creds (struct t_irc_server *server,
+                                       char **username,
+                                       char **password,
+                                       char **key);
 extern int irc_server_sasl_enabled (struct t_irc_server *server);
 extern char *irc_server_get_name_without_port (const char *name);
-extern void irc_server_set_addresses (struct t_irc_server *server,
-                                      const char *addresses);
-extern void irc_server_set_nicks (struct t_irc_server *server, const char *nicks);
-extern void irc_server_set_nick (struct t_irc_server *server, const char *nick);
+extern int irc_server_set_addresses (struct t_irc_server *server,
+                                     const char *addresses);
+extern void irc_server_set_nicks (struct t_irc_server *server,
+                                  const char *nicks);
+extern void irc_server_set_nick (struct t_irc_server *server,
+                                 const char *nick);
+extern void irc_server_set_host (struct t_irc_server *server,
+                                 const char *host);
 extern const char *irc_server_get_alternate_nick (struct t_irc_server *server);
 extern const char *irc_server_get_isupport_value (struct t_irc_server *server,
                                                   const char *feature);
+extern const char *irc_server_get_chantypes (struct t_irc_server *server);
 extern void irc_server_set_prefix_modes_chars (struct t_irc_server *server,
                                                const char *prefix);
+extern void irc_server_set_clienttagdeny (struct t_irc_server *server,
+                                          const char *clienttagdeny);
+extern void irc_server_set_lag (struct t_irc_server *server);
+extern void irc_server_set_tls_version (struct t_irc_server *server);
 extern const char *irc_server_get_prefix_modes (struct t_irc_server *server);
 extern const char *irc_server_get_prefix_chars (struct t_irc_server *server);
 extern int irc_server_get_prefix_mode_index (struct t_irc_server *server,
@@ -298,6 +353,11 @@ extern char irc_server_get_prefix_char_for_mode (struct t_irc_server *server,
 extern const char *irc_server_get_chanmodes (struct t_irc_server *server);
 extern int irc_server_prefix_char_statusmsg (struct t_irc_server *server,
                                              char prefix_char);
+extern int irc_server_get_max_modes (struct t_irc_server *server);
+extern char *irc_server_get_default_msg (const char *default_msg,
+                                         struct t_irc_server *server,
+                                         const char *channel_name,
+                                         const char *target_nick);
 extern struct t_irc_server *irc_server_alloc (const char *name);
 extern struct t_irc_server *irc_server_alloc_with_url (const char *irc_url);
 extern void irc_server_apply_command_line_options (struct t_irc_server *server,
@@ -307,10 +367,10 @@ extern struct t_irc_server *irc_server_copy (struct t_irc_server *server,
                                              const char *new_name);
 extern int irc_server_rename (struct t_irc_server *server, const char *new_name);
 extern int irc_server_reorder (const char **servers, int num_servers);
-extern void irc_server_send_signal (struct t_irc_server *server,
-                                    const char *signal, const char *command,
-                                    const char *full_message,
-                                    const char *tags);
+extern int irc_server_send_signal (struct t_irc_server *server,
+                                   const char *signal, const char *command,
+                                   const char *full_message,
+                                   const char *tags);
 extern void irc_server_set_send_default_tags (const char *tags);
 extern struct t_hashtable *irc_server_sendf (struct t_irc_server *server,
                                              int flags,
@@ -321,16 +381,16 @@ extern void irc_server_msgq_add_buffer (struct t_irc_server *server,
 extern void irc_server_msgq_flush ();
 extern void irc_server_set_buffer_title (struct t_irc_server *server);
 extern struct t_gui_buffer *irc_server_create_buffer (struct t_irc_server *server);
-#ifdef HAVE_GNUTLS
 int irc_server_fingerprint_search_algo_with_size (int size);
 char *irc_server_fingerprint_str_sizes ();
-#endif /* HAVE_GNUTLS */
 extern int irc_server_connect (struct t_irc_server *server);
 extern void irc_server_auto_connect (int auto_connect);
-extern void irc_server_autojoin_channels ();
-extern int irc_server_recv_cb (void *data, int fd);
-extern int irc_server_timer_sasl_cb (void *data, int remaining_calls);
-extern int irc_server_timer_cb (void *data, int remaining_calls);
+extern void irc_server_autojoin_channels (struct t_irc_server *server);
+extern int irc_server_recv_cb (const void *pointer, void *data, int fd);
+extern int irc_server_timer_sasl_cb (const void *pointer, void *data,
+                                     int remaining_calls);
+extern int irc_server_timer_cb (const void *pointer, void *data,
+                                int remaining_calls);
 extern void irc_server_outqueue_free_all (struct t_irc_server *server,
                                           int priority);
 extern int irc_server_get_channel_count (struct t_irc_server *server);
@@ -344,18 +404,26 @@ extern void irc_server_switch_address (struct t_irc_server *server,
 extern void irc_server_disconnect (struct t_irc_server *server,
                                    int switch_address, int reconnect);
 extern void irc_server_disconnect_all ();
+extern void irc_server_free_sasl_data (struct t_irc_server *server);
 extern void irc_server_free (struct t_irc_server *server);
-extern int irc_server_xfer_send_ready_cb (void *data, const char *signal,
-                                          const char *type_data, void *signal_data);
-extern int irc_server_xfer_resume_ready_cb (void *data, const char *signal,
-                                            const char *type_data, void *signal_data);
-extern int irc_server_xfer_send_accept_resume_cb (void *data, const char *signal,
+extern int irc_server_xfer_send_ready_cb (const void *pointer, void *data,
+                                          const char *signal,
+                                          const char *type_data,
+                                          void *signal_data);
+extern int irc_server_xfer_resume_ready_cb (const void *pointer, void *data,
+                                            const char *signal,
+                                            const char *type_data,
+                                            void *signal_data);
+extern int irc_server_xfer_send_accept_resume_cb (const void *pointer,
+                                                  void *data,
+                                                  const char *signal,
                                                   const char *type_data,
                                                   void *signal_data);
-extern struct t_hdata *irc_server_hdata_server_cb (void *data,
+extern struct t_hdata *irc_server_hdata_server_cb (const void *pointer,
+                                                   void *data,
                                                    const char *hdata_name);
 extern int irc_server_add_to_infolist (struct t_infolist *infolist,
                                        struct t_irc_server *server);
 extern void irc_server_print_log ();
 
-#endif /* WEECHAT_IRC_SERVER_H */
+#endif /* WEECHAT_PLUGIN_IRC_SERVER_H */
